@@ -128,52 +128,39 @@ public class ConstructNFA extends DepthFirstAdapter
 
   public void outARegExp(ARegExp node)
   {
-    ARegExpTail[] regExpTails = (ARegExpTail[]) node.getConcats().toArray(new ARegExpTail[0]);
+    AConcat[] concats = (AConcat[]) node.getConcats().toArray(new AConcat[0]);
     NFA result = null;
 
-    for(int i = regExpTails.length - 1; i >= 0 ; i--)
+    if(concats.length > 1)
     {
-      Object o = getOut(regExpTails[i]);
-      NFA nfa = (o instanceof NFA) ? (NFA) o : new NFA((CharSet) o);
+      for(int i = concats.length - 1; i >= 0 ; i--)
+      {
+        Object o = getOut(concats[i]);
+        NFA nfa = (o instanceof NFA) ? (NFA) o : new NFA((CharSet) o);
 
-      if(result == null)
-      {
-        result = nfa;
-      }
-      else
-      {
-        result = nfa.alternate(result);
+        if(result == null)
+        {
+          result = nfa;
+        }
+        else
+        {
+          result = nfa.alternate(result);
+        }
+
+        // free memory
+        if(getOut(concats[i]) != null)
+          setOut(concats[i], null);
       }
+      setOut(node, result);
+    }
+    else if(concats.length == 1)
+    {
+      setOut(node, getOut(concats[0]));
 
       // free memory
-      if(getOut(regExpTails[i]) != null)
-        setOut(regExpTails[i], null);
+      if(getOut(concats[0]) != null)
+        setOut(concats[0], null);
     }
-
-    if(result != null)
-    {
-      Object o = getOut(node.getConcat());
-      NFA nfa = (o instanceof NFA) ? (NFA) o : new NFA((CharSet) o);
-
-      setOut(node, nfa.alternate(result));
-    }
-    else
-    {
-      setOut(node, getOut(node.getConcat()));
-    }
-
-    // free memory
-    if(getOut(node.getConcat()) != null)
-      setOut(node.getConcat(), null);
-  }
-
-  public void outARegExpTail(ARegExpTail node)
-  {
-    setOut(node, getOut(node.getConcat()));
-
-    // free memory
-    if(getOut(node.getConcat()) != null)
-      setOut(node.getConcat(), null);
   }
 
   public void outAConcat(AConcat node)
