@@ -15,7 +15,7 @@ import java.io.*;
 public class GenAlts extends DepthFirstAdapter
 {
   private MacroExpander macros;
-  private ResolveIds ids;
+  private ResolveAstIds ast_ids;
   private File pkgDir;
   private String pkgName;
   private List elemList;
@@ -24,10 +24,9 @@ public class GenAlts extends DepthFirstAdapter
   ElemInfo info;
   //    final GenAlts instance = this;
 
-  public GenAlts(ResolveIds ids)
+  public GenAlts(ResolveAstIds ast_ids)
   {
-    this.ids = ids;
-
+    this.ast_ids = ast_ids;
     try
     {
       macros = new MacroExpander(
@@ -39,8 +38,8 @@ public class GenAlts extends DepthFirstAdapter
       throw new RuntimeException("unable to open alternatives.txt.");
     }
 
-    pkgDir = new File(ids.pkgDir, "node");
-    pkgName = ids.pkgName.equals("") ? "node" : ids.pkgName + ".node";
+    pkgDir = new File(ast_ids.astIds.pkgDir, "node");
+    pkgName = ast_ids.astIds.pkgName.equals("") ? "node" : ast_ids.astIds.pkgName + ".node";
 
     if(!pkgDir.exists())
     {
@@ -51,32 +50,24 @@ public class GenAlts extends DepthFirstAdapter
     }
   }
 
-  public void inAProd(AProd node)
+  public void inAAstProd(AAstProd node)
   {
-    currentProd = (String) ids.names.get(node);
+    currentProd = (String) ast_ids.ast_names.get(node);
   }
 
-  public void inAParsedAlt(AParsedAlt node)
-  {
-    inPAlt(node);
-  }
-
-  public void inAIgnoredAlt(AIgnoredAlt node)
-  {
-    inPAlt(node);
-  }
-
-  public void inPAlt(PAlt node)
+  public void inAAstAlt(AAstAlt node)
   {
     elemList = new TypedLinkedList(ElemInfoCast.instance);
   }
 
+  public void caseAProductions(AProductions node)
+  {}
+
   public void inAElem(AElem node)
   {
     info = new ElemInfo();
-
-    info.name = (String) ids.names.get(node);
-    info.type = (String) ids.elemTypes.get(node);
+    info.name = (String) ast_ids.ast_names.get(node);
+    info.type = (String) ast_ids.ast_elemTypes.get(node);
     info.operator = ElemInfo.NONE;
 
     if(node.getUnOp() != null)
@@ -100,24 +91,13 @@ public class GenAlts extends DepthFirstAdapter
                            }
                           );
     }
-
     elemList.add(info);
     info = null;
   }
 
-  public void outAParsedAlt(AParsedAlt node)
+  public void outAAstAlt(AAstAlt node)
   {
-    outPAlt(node);
-  }
-
-  public void outAIgnoredAlt(AIgnoredAlt node)
-  {
-    outPAlt(node);
-  }
-
-  public void outPAlt(PAlt node)
-  {
-    String name = (String) ids.names.get(node);
+    String name = (String) ast_ids.ast_names.get(node);
 
     BufferedWriter file;
 
@@ -134,35 +114,58 @@ public class GenAlts extends DepthFirstAdapter
 
     try
     {
-      boolean hasOperator = false;
-
-      macros.apply(file, "AlternativeHeader", new String[] {pkgName,
-                   ids.pkgName.equals("") ? "analysis" : ids.pkgName + ".analysis",
-                   name, currentProd});
+        boolean hasOperator = false;
+        boolean hasList = false;
 
       for(Iterator i = elemList.iterator(); i.hasNext();)
       {
         ElemInfo info = (ElemInfo) i.next();
+        if(info != null)
+          switch(info.operator)
+          {
+          case ElemInfo.STAR:
+          case ElemInfo.PLUS:
+            {
+              hasList = true;
+            }
+            break;
+          }
+      }
 
-        switch(info.operator)
-        {
-        case ElemInfo.QMARK:
-        case ElemInfo.NONE:
+      macros.apply(file, "AlternativeHeader1", new String[] {pkgName});
+
+      if(hasList)
+      {
+          macros.apply(file, "AlternativeHeaderList", new String[] {});
+      }
+
+      macros.apply(file, "AlternativeHeader2", new String[] {
+              ast_ids.astIds.pkgName.equals("") ? "analysis" : ast_ids.astIds.pkgName + ".analysis",
+              name, currentProd});
+
+      for(Iterator i = elemList.iterator(); i.hasNext();)
+      {
+        ElemInfo info = (ElemInfo) i.next();
+        if(info != null)
+          switch(info.operator)
           {
-            macros.apply(file, "NodeElement",
-                         new String[] {info.type,
-                                       nodeName(info.name)});
+          case ElemInfo.QMARK:
+          case ElemInfo.NONE:
+            {
+              macros.apply(file, "NodeElement",
+                           new String[] {info.type,
+                                         nodeName(info.name)});
+            }
+            break;
+          case ElemInfo.STAR:
+          case ElemInfo.PLUS:
+            {
+              hasOperator = true;
+              macros.apply(file, "ListElement",
+                           new String[] {info.type, nodeName(info.name)});
+            }
+            break;
           }
-          break;
-        case ElemInfo.STAR:
-        case ElemInfo.PLUS:
-          {
-            hasOperator = true;
-            macros.apply(file, "ListElement",
-                         new String[] {info.name, nodeName(info.name)});
-          }
-          break;
-        }
       }
 
       macros.apply(file, "ConstructorHeader",
@@ -178,24 +181,24 @@ public class GenAlts extends DepthFirstAdapter
         for(Iterator i = elemList.iterator(); i.hasNext();)
         {
           ElemInfo info = (ElemInfo) i.next();
-
-          switch(info.operator)
-          {
-          case ElemInfo.QMARK:
-          case ElemInfo.NONE:
+          if(info != null)
+            switch(info.operator)
             {
-              macros.apply(file, "ConstructorHeaderDeclNode",
-                           new String[] {info.type, nodeName(info.name), i.hasNext() ? "," : ""});
+            case ElemInfo.QMARK:
+            case ElemInfo.NONE:
+              {
+                macros.apply(file, "ConstructorHeaderDeclNode",
+                             new String[] {info.type, nodeName(info.name), i.hasNext() ? "," : ""});
+              }
+              break;
+            case ElemInfo.STAR:
+            case ElemInfo.PLUS:
+              {
+                macros.apply(file, "ConstructorHeaderDeclList",
+                             new String[] {info.type, nodeName(info.name), i.hasNext() ? "," : ""});
+              }
+              break;
             }
-            break;
-          case ElemInfo.STAR:
-          case ElemInfo.PLUS:
-            {
-              macros.apply(file, "ConstructorHeaderDeclList",
-                           new String[] {"List", nodeName(info.name), i.hasNext() ? "," : ""});
-            }
-            break;
-          }
         }
 
         macros.apply(file, "ConstructorBodyHeader", null);
@@ -204,80 +207,24 @@ public class GenAlts extends DepthFirstAdapter
         {
           ElemInfo info = (ElemInfo) i.next();
 
-          switch(info.operator)
-          {
-          case ElemInfo.QMARK:
-          case ElemInfo.NONE:
+          if(info != null )
+            switch(info.operator)
             {
-              macros.apply(file, "ConstructorBodyNode",
-                           new String[] {info.name, nodeName(info.name)});
+            case ElemInfo.QMARK:
+            case ElemInfo.NONE:
+              {
+                macros.apply(file, "ConstructorBodyNode",
+                             new String[] {info.name, nodeName(info.name)});
+              }
+              break;
+            case ElemInfo.STAR:
+            case ElemInfo.PLUS:
+              {
+                macros.apply(file, "ConstructorBodyList",
+                             new String[] {info.name, nodeName(info.name)});
+              }
+              break;
             }
-            break;
-          case ElemInfo.STAR:
-          case ElemInfo.PLUS:
-            {
-              macros.apply(file, "ConstructorBodyList",
-                           new String[] {nodeName(info.name)});
-            }
-            break;
-          }
-        }
-
-        macros.apply(file, "ConstructorBodyTail", null);
-      }
-
-      if(hasOperator)
-      {
-        macros.apply(file, "ConstructorHeader",
-                     new String[] {name});
-
-        for(Iterator i = elemList.iterator(); i.hasNext();)
-        {
-          ElemInfo info = (ElemInfo) i.next();
-
-          switch(info.operator)
-          {
-          case ElemInfo.NONE:
-          case ElemInfo.QMARK:
-            {
-              macros.apply(file, "ConstructorHeaderDeclNode",
-                           new String[] {info.type, nodeName(info.name), i.hasNext() ? "," : ""});
-            }
-            break;
-          case ElemInfo.STAR:
-          case ElemInfo.PLUS:
-            {
-              macros.apply(file, "ConstructorHeaderDeclList",
-                           new String[] {"X" + info.type,
-                                         nodeName(info.name), i.hasNext() ? "," : ""});
-            }
-            break;
-          }
-        }
-
-        macros.apply(file, "ConstructorBodyHeader", null);
-
-        for(Iterator i = elemList.iterator(); i.hasNext();)
-        {
-          ElemInfo info = (ElemInfo) i.next();
-
-          switch(info.operator)
-          {
-          case ElemInfo.NONE:
-          case ElemInfo.QMARK:
-            {
-              macros.apply(file, "ConstructorBodyNode",
-                           new String[] {info.name, nodeName(info.name)});
-            }
-            break;
-          case ElemInfo.STAR:
-          case ElemInfo.PLUS:
-            {
-              macros.apply(file, "ConstructorBodyPlus",
-                           new String[] {nodeName(info.name), info.type});
-            }
-            break;
-          }
         }
 
         macros.apply(file, "ConstructorBodyTail", null);
@@ -290,28 +237,27 @@ public class GenAlts extends DepthFirstAdapter
       for(Iterator i = elemList.iterator(); i.hasNext();)
       {
         ElemInfo info = (ElemInfo) i.next();
-
-        switch(info.operator)
-        {
-        case ElemInfo.QMARK:
-        case ElemInfo.NONE:
+        if(info != null)
+          switch(info.operator)
           {
-            macros.apply(file, "CloneBodyNode",
-                         new String[] {info.type, nodeName(info.name), i.hasNext() ? "," : ""});
+          case ElemInfo.QMARK:
+          case ElemInfo.NONE:
+            {
+              macros.apply(file, "CloneBodyNode",
+                           new String[] {info.type, nodeName(info.name), i.hasNext() ? "," : ""});
+            }
+            break;
+          case ElemInfo.STAR:
+          case ElemInfo.PLUS:
+            {
+              macros.apply(file, "CloneBodyList",
+                           new String[] {nodeName(info.name), i.hasNext() ? "," : ""});
+            }
+            break;
           }
-          break;
-        case ElemInfo.STAR:
-        case ElemInfo.PLUS:
-          {
-            macros.apply(file, "CloneBodyList",
-                         new String[] {nodeName(info.name), i.hasNext() ? "," : ""});
-          }
-          break;
-        }
       }
 
       macros.apply(file, "CloneTail", null);
-      //****************
 
       macros.apply(file, "Apply", new String[] {name});
 
@@ -319,23 +265,24 @@ public class GenAlts extends DepthFirstAdapter
       {
         ElemInfo info = (ElemInfo) i.next();
 
-        switch(info.operator)
-        {
-        case ElemInfo.QMARK:
-        case ElemInfo.NONE:
+        if(info != null)
+          switch(info.operator)
           {
-            macros.apply(file, "GetSetNode",
-                         new String[] {info.type, info.name, nodeName(info.name)});
+          case ElemInfo.QMARK:
+          case ElemInfo.NONE:
+            {
+              macros.apply(file, "GetSetNode",
+                           new String[] {info.type, info.name, nodeName(info.name)});
+            }
+            break;
+          case ElemInfo.STAR:
+          case ElemInfo.PLUS:
+            {
+              macros.apply(file, "GetSetList",
+                           new String[] {info.name, nodeName(info.name), info.type});
+            }
+            break;
           }
-          break;
-        case ElemInfo.STAR:
-        case ElemInfo.PLUS:
-          {
-            macros.apply(file, "GetSetList",
-                         new String[] {info.name, nodeName(info.name)});
-          }
-          break;
-        }
       }
 
       macros.apply(file, "ToStringHeader", null);
@@ -343,23 +290,24 @@ public class GenAlts extends DepthFirstAdapter
       {
         ElemInfo info = (ElemInfo) i.next();
 
-        switch(info.operator)
-        {
-        case ElemInfo.QMARK:
-        case ElemInfo.NONE:
+        if(info != null)
+          switch(info.operator)
           {
-            macros.apply(file, "ToStringBodyNode",
-                         new String[] {nodeName(info.name)});
+          case ElemInfo.QMARK:
+          case ElemInfo.NONE:
+            {
+              macros.apply(file, "ToStringBodyNode",
+                           new String[] {nodeName(info.name)});
+            }
+            break;
+          case ElemInfo.STAR:
+          case ElemInfo.PLUS:
+            {
+              macros.apply(file, "ToStringBodyList",
+                           new String[] {nodeName(info.name)});
+            }
+            break;
           }
-          break;
-        case ElemInfo.STAR:
-        case ElemInfo.PLUS:
-          {
-            macros.apply(file, "ToStringBodyList",
-                         new String[] {nodeName(info.name)});
-          }
-          break;
-        }
       }
       macros.apply(file, "ToStringTail", null);
 
@@ -368,68 +316,53 @@ public class GenAlts extends DepthFirstAdapter
       {
         ElemInfo info = (ElemInfo) i.next();
 
-        switch(info.operator)
-        {
-        case ElemInfo.QMARK:
-        case ElemInfo.NONE:
+        if(info != null)
+          switch(info.operator)
           {
-            macros.apply(file, "RemoveChildNode",
-                         new String[] {nodeName(info.name)});
+          case ElemInfo.QMARK:
+          case ElemInfo.NONE:
+            {
+              macros.apply(file, "RemoveChildNode",
+                           new String[] {nodeName(info.name)});
+            }
+            break;
+          case ElemInfo.STAR:
+          case ElemInfo.PLUS:
+            {
+              macros.apply(file, "RemoveChildList",
+                           new String[] {nodeName(info.name)});
+            }
+            break;
           }
-          break;
-        case ElemInfo.STAR:
-        case ElemInfo.PLUS:
-          {
-            macros.apply(file, "RemoveChildList",
-                         new String[] {nodeName(info.name)});
-          }
-          break;
-        }
       }
       macros.apply(file, "RemoveChildTail", null);
-      /* */
+
       macros.apply(file, "ReplaceChildHeader", null);
       for(Iterator i = elemList.iterator(); i.hasNext();)
       {
         ElemInfo info = (ElemInfo) i.next();
 
-        switch(info.operator)
-        {
-        case ElemInfo.QMARK:
-        case ElemInfo.NONE:
+        if(info != null)
+          switch(info.operator)
           {
-            macros.apply(file, "ReplaceChildNode",
-                         new String[] {nodeName(info.name), info.name, info.type});
+          case ElemInfo.QMARK:
+          case ElemInfo.NONE:
+            {
+              macros.apply(file, "ReplaceChildNode",
+                           new String[] {nodeName(info.name), info.name, info.type});
+            }
+            break;
+          case ElemInfo.STAR:
+          case ElemInfo.PLUS:
+            {
+              macros.apply(file, "ReplaceChildList",
+                           new String[] {nodeName(info.name), info.type});
+            }
+            break;
           }
-          break;
-        case ElemInfo.STAR:
-        case ElemInfo.PLUS:
-          {
-            macros.apply(file, "ReplaceChildList",
-                         new String[] {nodeName(info.name)});
-          }
-          break;
-        }
       }
       macros.apply(file, "ReplaceChildTail", null);
 
-      /* */
-      for(Iterator i = elemList.iterator(); i.hasNext();)
-      {
-        ElemInfo info = (ElemInfo) i.next();
-
-        switch(info.operator)
-        {
-        case ElemInfo.STAR:
-        case ElemInfo.PLUS:
-          {
-            macros.apply(file, "Cast",
-                         new String[] {info.name, info.type, name});
-          }
-          break;
-        }
-      }
-
       macros.apply(file, "AlternativeTail", null);
     }
     catch(IOException e)
@@ -485,4 +418,3 @@ public class GenAlts extends DepthFirstAdapter
     }
   }
 }
-
