@@ -15,7 +15,7 @@ import java.io.*;
 public class GenAnalyses extends DepthFirstAdapter
 {
   private MacroExpander macros;
-  private ResolveIds ids;
+  private ResolveAstIds ast_ids;
   private File pkgDir;
   private String pkgName;
   private List elemList;
@@ -26,10 +26,9 @@ public class GenAnalyses extends DepthFirstAdapter
   ElemInfo info;
   //    final GenAnalyses instance = this;
 
-  public GenAnalyses(ResolveIds ids)
+  public GenAnalyses(ResolveAstIds ast_ids)
   {
-    this.ids = ids;
-
+    this.ast_ids = ast_ids;
     try
     {
       macros = new MacroExpander(
@@ -41,8 +40,8 @@ public class GenAnalyses extends DepthFirstAdapter
       throw new RuntimeException("unable to open analyses.txt.");
     }
 
-    pkgDir = new File(ids.pkgDir, "analysis");
-    pkgName = ids.pkgName.equals("") ? "analysis" : ids.pkgName + ".analysis";
+    pkgDir = new File(ast_ids.astIds.pkgDir, "analysis");
+    pkgName = ast_ids.astIds.pkgName.equals("") ? "analysis" : ast_ids.astIds.pkgName + ".analysis";
 
     if(!pkgDir.exists())
     {
@@ -53,46 +52,40 @@ public class GenAnalyses extends DepthFirstAdapter
     }
   }
 
-  public void inAProd(AProd node)
+  public void inAAstProd(AAstProd node)
   {
     if(mainProduction == null)
     {
-      mainProduction = (String) ids.names.get(node);
+      mainProduction = (String) ast_ids.ast_names.get(node);
     }
   }
 
   public void inATokenDef(ATokenDef node)
   {
-    tokenList.add(ids.names.get(node));
+    tokenList.add(ast_ids.astIds.names.get(node));
   }
 
-  public void inAParsedAlt(AParsedAlt node)
-  {
-    inPAlt(node);
-  }
-
-  public void inAIgnoredAlt(AIgnoredAlt node)
-  {
-    inPAlt(node);
-  }
-
-  public void inPAlt(PAlt node)
+  public void inAAstAlt(AAstAlt node)
   {
     elemList = new TypedLinkedList(ElemInfoCast.instance);
   }
 
+  public void caseAProductions(AProductions node)
+  {}
+
   public void inAElem(AElem node)
   {
     info = new ElemInfo();
 
-    info.name = (String) ids.names.get(node);
-    info.type = (String) ids.elemTypes.get(node);
+    info.name = (String) ast_ids.ast_names.get(node);
+    info.type = (String) ast_ids.ast_elemTypes.get(node);
     info.operator = ElemInfo.NONE;
 
     if(node.getUnOp() != null)
     {
       node.getUnOp().apply(new DepthFirstAdapter()
                            {
+
                              public void caseAStarUnOp(AStarUnOp node)
                              {
                                info.operator = ElemInfo.STAR;
@@ -115,21 +108,11 @@ public class GenAnalyses extends DepthFirstAdapter
     info = null;
   }
 
-  public void outAParsedAlt(AParsedAlt node)
-  {
-    outPAlt(node);
-  }
-
-  public void outAIgnoredAlt(AIgnoredAlt node)
-  {
-    outPAlt(node);
-  }
-
-  public void outPAlt(PAlt node)
+  public void outAAstAlt(AAstAlt node)
   {
     AltInfo info = new AltInfo();
 
-    info.name = (String) ids.names.get(node);
+    info.name = (String) ast_ids.ast_names.get(node);
     info.elems.addAll(elemList);
     elemList = null;
 
@@ -166,7 +149,7 @@ public class GenAnalyses extends DepthFirstAdapter
     try
     {
       macros.apply(file, "AnalysisHeader", new String[] {pkgName,
-                   ids.pkgName.equals("") ? "node" : ids.pkgName + ".node"});
+                   ast_ids.astIds.pkgName.equals("") ? "node" : ast_ids.astIds.pkgName + ".node"});
 
       if(mainProduction != null)
       {
@@ -223,7 +206,7 @@ public class GenAnalyses extends DepthFirstAdapter
     try
     {
       macros.apply(file, "AnalysisAdapterHeader", new String[] {pkgName,
-                   ids.pkgName.equals("") ? "node" : ids.pkgName + ".node"});
+                   ast_ids.astIds.pkgName.equals("") ? "node" : ast_ids.astIds.pkgName + ".node"});
 
       if(mainProduction != null)
       {
@@ -278,7 +261,7 @@ public class GenAnalyses extends DepthFirstAdapter
     try
     {
       macros.apply(file, "DepthFirstAdapterHeader", new String[] {pkgName,
-                   ids.pkgName.equals("") ? "node" : ids.pkgName + ".node",
+                   ast_ids.astIds.pkgName.equals("") ? "node" : ast_ids.astIds.pkgName + ".node",
                    mainProduction});
 
       for(Iterator i = altList.iterator(); i.hasNext();)
@@ -353,7 +336,7 @@ public class GenAnalyses extends DepthFirstAdapter
     try
     {
       macros.apply(file, "ReversedDepthFirstAdapterHeader", new String[] {pkgName,
-                   ids.pkgName.equals("") ? "node" : ids.pkgName + ".node",
+                   ast_ids.astIds.pkgName.equals("") ? "node" : ast_ids.astIds.pkgName + ".node",
                    mainProduction});
 
       for(Iterator i = altList.iterator(); i.hasNext();)
@@ -454,4 +437,3 @@ public class GenAnalyses extends DepthFirstAdapter
     }
   }
 }
-
