@@ -15,30 +15,87 @@ import org.sablecc.sablecc.Grammar;
 import java.util.Vector;
 import java.util.Enumeration;
 
+/*
+ * GenParser
+ * 
+ * This class is the responsible of generation of the parser.
+ * It calls another classes which will do internal transformations
+ * to the grammar in order to made it support by BNF parser generator
+ * algorithm whereas SableCC3.x.x grammars can be written with some 
+ * operators of the EBNF form.
+ * It can also call an appropriate Tree-Walker which in case of conflict
+ * will try to inline productions involved in the conflict with the aim of
+ * resolving it.
+ */
+
 public class GenParser extends DepthFirstAdapter
 {
+  //This is the tree-walker field which made internal transformations("EBNF"->BNF) to the grammar
+  InternalTransformationsToGrammar bnf_and_CST_AST_Transformations;
+
+  //tree-walker field which construct data structures for generating of parsing tables.
+  ConstructParserGenerationDatas genParserAdapter;
+
   private MacroExpander macros;
   private ResolveIds ids;
+  private ResolveAltIds altIds;
+  private ResolveTransformIds transformIds;
+
+  // This class reference variable fills the map "altsElemTypes" from class AlternativeElementTypes. It associates
+  // the name of elements with its types
+  private AlternativeElementTypes AET;
+
+  //This computes variables declarations position and type for parser generation.
+  //In fact it helps to determine how many elements are needed to pop from the stack
+  private ComputeCGNomenclature CG;
+
+  //This helps to compute alternative transformations code generation.
+  private ComputeSimpleTermPosition CTP;
+
   private File pkgDir;
   private String pkgName;
-  private String currentProd;
-  private int currentAlt;
   private boolean hasProductions;
+  private String firstProductionName;
+  private boolean processInlining;
+  private boolean prettyPrinting;
+  private boolean grammarHasTransformations;
 
-  static final int NONE = 0;
-  static final int STAR = 1;
-  static final int QMARK = 2;
-  static final int PLUS = 3;
-  int count;
-  int elem;
-  Map alts = new TypedTreeMap(
-               StringComparator.instance,
-               StringCast.instance,
-               NodeCast.instance);
+  // This boolean is used to check weither the filter() method in class Parser.java
+  // should be present or not.
+  private boolean activateFilter = true;
 
-  public GenParser(ResolveIds ids)
+  //This tree-walker field generate the code of parsing and construction of the AST.
+  GenerateAlternativeCodeForParser aParsedAltAdapter;
+
+  private LinkedList listSimpleTermTransform = new LinkedList();
+
+  public final Map simpleTermTransform =
+    new TypedHashMap(NodeCast.instance,
+                     StringCast.instance);
+
+  //This map contains Productions which were explicitely transformed in the grammar
+  //Those transformations was specified by the grammar-writer.
+  private final Map mapProductionTransformations =
+    new TypedHashMap(StringCast.instance,
+                     ListCast.instance);
+
+  private Map alts;
+
+  public GenParser(ResolveIds ids, ResolveAltIds altIds, ResolveTransformIds transformIds,
+                   String firstProductionName, boolean processInlining, boolean prettyPrinting,
+                   boolean grammarHasTransformations)
   {
     this.ids = ids;
+    this.altIds = altIds;
+    this.transformIds = transformIds;
+    this.processInlining = processInlining;
+    this.prettyPrinting = prettyPrinting;
+    this.grammarHasTransformations = grammarHasTransformations;
+
+    AET = new AlternativeElementTypes(ids);
+    CG = new ComputeCGNomenclature(ids, transformIds.getProdTransformIds());
+    CTP = new ComputeSimpleTermPosition(ids);
+    this.firstProductionName = firstProductionName;
 
     try
     {
@@ -65,49 +122,137 @@ public class GenParser extends DepthFirstAdapter
 
   public void caseStart(Start tree)
   {
-    tree.apply(new DepthFirstAdapter()
-               {
-                 private boolean hasAlternative;
-
-                 public void caseATokenDef(ATokenDef node)
-                 {
-                   String name = (String) ids.names.get(node);
-                   String errorName = (String) ids.errorNames.get(node);
-
-                   if(!ids.ignTokens.containsKey(name))
-                   {
-                     Grammar.addTerminal(name, errorName);
-                   }
-                 }
-
-                 public void inAProd(AProd node)
-                 {
-                   hasAlternative = false;
-                 }
-
-                 public void inAParsedAlt(AParsedAlt node)
-                 {
-                   hasAlternative = true;
-                 }
-
-                 public void outAProd(AProd node)
-                 {
-                   if(hasAlternative)
-                   {
-                     Grammar.addNonterminal((String) ids.names.get(node));
-                   }
-                 }
-               }
-              );
-
-    tree.getPGrammar().apply(this);
+    tree.getPGrammar().apply(new DepthFirstAdapter()
+                             {
+                               public void caseAProd(AProd node)
+                               {
+                                 hasProductions = true;
+                                 if(node.getProdTransform() != null)
+                                 {
+                                   mapProductionTransformations.put("P"+ResolveIds.name(node.getId().getText()),
+                                                                    node.getProdTransform().clone() );
+                                 }
+                               }
+                             }
+                            );
 
     if(!hasProductions)
     {
       return;
     }
 
-    Grammar.computeLALR();
+    //Performing internal transformations
+    bnf_and_CST_AST_Transformations =
+      new InternalTransformationsToGrammar(ids, altIds, transformIds,
+                                           listSimpleTermTransform,
+                                           simpleTermTransform,
+                                           mapProductionTransformations,
+                                           transformIds.simpleTermOrsimpleListTermTypes);
+
+    //apply internal transformations to the grammar.
+    tree.getPGrammar().apply(bnf_and_CST_AST_Transformations);
+
+    if(prettyPrinting)
+    {
+      tree.apply(new PrettyPrinter());
+      return;
+    }
+
+    ConstructProdsMap mapOfProds = new ConstructProdsMap();
+    tree.apply(mapOfProds);
+
+    boolean computeLALR = false;
+
+    //This do-while loop is managing the inlining process.
+    do
+    {
+      //Initialization of parsing tables and some symbol tables
+      //names and elemTypes from ResolveIds.
+      reinit();
+      reConstructSymbolTables(tree);
+
+      tree.apply(new DepthFirstAdapter()
+                 {
+                   private boolean hasAlternative;
+
+                   public void caseATokenDef(ATokenDef node)
+                   {
+                     String name = (String) ids.names.get(node);
+                     String errorName = (String) ids.errorNames.get(node);
+
+                     if(!ids.ignTokens.containsKey(name))
+                     {
+                       Grammar.addTerminal(name, errorName);
+                     }
+                   }
+
+                   public void inAProd(AProd node)
+                   {
+                     hasAlternative = false;
+                   }
+
+                   public void inAAlt(AAlt node)
+                   {
+                     hasAlternative = true;
+                   }
+
+                   public void outAProd(AProd node)
+                   {
+                     if(hasAlternative)
+                     {
+                       Grammar.addNonterminal((String) ids.names.get(node));
+                     }
+                   }
+                 }
+                );
+
+      //Construct all necessary informations for generation of the parser.
+      //This map contains all the alternatives of the transformed final grammar
+      alts = new TypedHashMap(StringCast.instance, NodeCast.instance);
+
+      tree.getPGrammar().apply(new ConstructParserGenerationDatas(ids, alts));
+
+      try
+      {
+        //Generation of parsing symbol tables
+        Grammar.computeLALR();
+        computeLALR = true;
+      }
+      catch(ConflictException ce)
+      {
+        if(activateFilter)
+        {
+          activateFilter = false;
+        }
+
+        //Here, we are trying to inline the grammar with production imply in the conflict.
+        if(processInlining)
+        {
+          ComputeInlining grammarToBeInlinedWithConflictualProductions = new ComputeInlining(ce.getConflictualProductions(),
+              mapOfProds.productionsMap,
+              tree);
+          if(!grammarToBeInlinedWithConflictualProductions.computeInlining())
+          {
+            System.out.println("\nA previous conflict that we've tried to solve by inline some productions inside the grammars cannot be solved that way. The transformed grammar is : ");
+            tree.apply(new PrettyPrinter());
+            throw new RuntimeException(ce.getMessage());
+          }
+
+          System.out.println();
+          System.out.println("Inlining.");
+        }
+        else
+        {
+          throw new RuntimeException(ce.getMessage());
+        }
+      }
+    }
+    while(!computeLALR);
+
+    tree.getPGrammar().apply(AET);
+    CG.setAltElemTypes(AET.getMapOfAltElemType());
+    tree.getPGrammar().apply(CG);
+    tree.getPGrammar().apply(CTP);
 
     createParser();
     createParserException();
@@ -115,163 +260,95 @@ public class GenParser extends DepthFirstAdapter
     createTokenIndex();
   }
 
-  public void inAProd(AProd node)
+  public void reinit()
   {
-    hasProductions = true;
-    currentProd = (String) ids.names.get(node);
+    // re-initialize all static structures in the engine
+    LR0Collection.reinit();
+    Symbol.reinit();
+    Production.reinit();
+    Grammar.reinit();
+    ids.reinit();
   }
 
-  public void caseAParsedAlt(AParsedAlt node)
+  private String currentProd;
+  private String currentAlt;
+
+  //reconstruction of map names of class ResolveIds
+  public void reConstructSymbolTables(Start tree)
   {
-    count = 1;
-
-    node.apply(new DepthFirstAdapter()
+    tree.apply(new DepthFirstAdapter()
                {
-                 public void inAElem(AElem node)
+                 public void caseAProd(AProd node)
                  {
-                   GenParser.this.setOut(node, new Integer(NONE));
+                   currentProd = ids.name(node.getId().getText());
+                   String name = "P" + currentProd;
+
+                   ids.names.put(node, name);
+
+                   //list of inAAlt code.
+                   Object []list_alt = (Object [])node.getAlts().toArray();
+                   for(int i = 0; i< list_alt.length; i++)
+                   {
+                     ((PAlt)list_alt[i]).apply(this);
+                   }
                  }
 
-                 public void caseAStarUnOp(AStarUnOp node)
+                 public void outAHelperDef(AHelperDef node)
                  {
-                   count *= 2;
-
-                   GenParser.this.setOut(node.parent(), new Integer(STAR));
+                   String name = node.getId().getText();
+                   ids.names.put(node, name);
                  }
 
-                 public void caseAQMarkUnOp(AQMarkUnOp node)
+                 public void outATokenDef(ATokenDef node)
                  {
-                   count *= 2;
+                   String name = "T" + ids.name(node.getId().getText());
 
-                   GenParser.this.setOut(node.parent(), new Integer(QMARK));
+                   ids.names.put(node, name);
                  }
 
-                 public void caseAPlusUnOp(APlusUnOp node)
+                 public void caseAAlt(final AAlt alt)
                  {
-                   GenParser.this.setOut(node.parent(), new Integer(PLUS));
+                   if(alt.getAltName() != null)
+                   {
+                     currentAlt =
+                       "A" +
+                       ids.name(alt.getAltName().getText()) +
+                       currentProd;
+
+                     ids.names.put(alt, currentAlt);
+                   }
+                   else
+                   {
+                     currentAlt = "A" + currentProd;
+                     ids.names.put(alt, currentAlt);
+                   }
+
+                   AElem list_elem[] = (AElem[]) alt.getElems().toArray(new AElem[0]);
+                   for(int i=0; i<list_elem.length;i++)
+                   {
+                     list_elem[i].apply(this);
+                   }
+                 }
+
+                 public void caseAAst(AAst node)
+                 {}
+
+                 public void caseAElem(final AElem elem)
+                 {
+                   if(elem.getElemName() != null)
+                   {
+                     ids.names.put(elem, ids.name(elem.getElemName().getText()) );
+                   }
+                   else
+                   {
+                     ids.names.put(elem, ids.name(elem.getId().getText()));
+                   }
                  }
                }
               );
-
-    if(count == 1)
-    {
-      alts.put(ids.names.get(node), node);
-
-      currentAlt = Grammar.addProduction(currentProd, (String) ids.names.get(node));
-
-      {
-        Object temp[] = node.getElems().toArray();
-        for(int i = 0; i < temp.length; i++)
-        {
-          ((PElem) temp[i]).apply(this);
-        }
-      }
-    }
-    else
-    {
-      int max = count;
-
-      for(count = 0; count < max; count++)
-      {
-        elem = 0;
-
-        alts.put(
-          "X" + (count + 1) + (String) ids.names.get(node),
-          node);
-
-        currentAlt = Grammar.addProduction(currentProd,
-                                           "X" + (count + 1) + (String) ids.names.get(node));
-
-        {
-          Object temp[] = node.getElems().toArray();
-          for(int i = 0; i < temp.length; i++)
-          {
-            ((PElem) temp[i]).apply(this);
-          }
-        }
-      }
-    }
   }
 
-  public void caseAElem(AElem node)
-  {
-    int op = ((Integer) getOut(node)).intValue();
-    String name = (String) ids.elemTypes.get(node);
-
-    switch(op)
-    {
-    case NONE:
-      {
-        Grammar.addSymbolToProduction(name, currentAlt);
-      }
-      break;
-    case STAR:
-      {
-        //                System.out.println("Star:" + count + ", " + (1 << elem));
-
-        if((count & (1 << elem)) != 0)
-        {
-          //                    System.out.println("yes");
-
-          try
-          {
-            Grammar.addNonterminal("X" + name);
-
-            int alt = Grammar.addProduction("X" + name, "X1" + name);
-
-            Grammar.addSymbolToProduction("X" + name, alt);
-            Grammar.addSymbolToProduction(name, alt);
-
-            alt = Grammar.addProduction("X" + name, "X2" + name);
-
-            Grammar.addSymbolToProduction(name, alt);
-          }
-          catch(Exception e)
-          {}
-
-          Grammar.addSymbolToProduction("X" + name, currentAlt);
-        }
-
-        elem++;
-      }
-      break;
-    case QMARK:
-      {
-        if((count & (1 << elem)) != 0)
-        {
-          Grammar.addSymbolToProduction(name, currentAlt);
-        }
-
-        elem++;
-      }
-      break;
-    case PLUS:
-      {
-        try
-        {
-          Grammar.addNonterminal("X" + name);
-
-          int alt = Grammar.addProduction("X" + name, "X1" + name);
-
-          Grammar.addSymbolToProduction("X" + name, alt);
-          Grammar.addSymbolToProduction(name, alt);
-
-          alt = Grammar.addProduction("X" + name, "X2" + name);
-
-          Grammar.addSymbolToProduction(name, alt);
-        }
-        catch(Exception e)
-        {}
-
-        Grammar.addSymbolToProduction("X" + name, currentAlt);
-      }
-      break;
-    }
-  }
-
-  public void caseAIgnoredAlt(AIgnoredAlt node)
-  {}
-
+  //Parser.java Generation
   private void createParser()
   {
     BufferedWriter file;
@@ -298,170 +375,92 @@ public class GenParser extends DepthFirstAdapter
                    ids.pkgName.equals("") ? "node" : ids.pkgName + ".node",
                    ids.pkgName.equals("") ? "analysis" : ids.pkgName + ".analysis"});
 
+      if(activateFilter && !grammarHasTransformations)
+      {
+        macros.apply(file, "ParserNoInliningPushHeader");
+        macros.apply(file, "ParserCommon", new String[] {", true", ", false"});
+      }
+      else
+      {
+        macros.apply(file, "ParserInliningPushHeader");
+        macros.apply(file, "ParserCommon", new String[] {"", ""});
+      }
+
+      //this loop generates the code for all possible reductions and the type of
+      //the node needed to be created at a local point.
       for(int i = 0; i < (productions.length - 1); i++)
       {
         Node node = (Node) alts.get(productions[i].name);
 
-        if(node == null)
+        if(activateFilter && !grammarHasTransformations)
         {
-          macros.apply(file, "ParserReduceNoFilter", new String[] {
+          macros.apply(file, "ParserNoInliningReduce", new String[] {
                          "" + productions[i].index,
-                         "" + productions[i].leftside});
+                         "" + productions[i].leftside,
+                         "" + (productions[i].name.startsWith("ANonTerminal$") ||
+                               productions[i].name.startsWith("ATerminal$")),
+                         productions[i].name});
         }
         else
         {
-          macros.apply(file, "ParserReduceFilter", new String[] {
+          macros.apply(file, "ParserInliningReduce", new String[] {
                          "" + productions[i].index,
-                         "" + productions[i].leftside});
+                         "" + productions[i].leftside,
+                         productions[i].name});
         }
       }
 
-      {
-        Symbol[] rightside = productions[productions.length - 1].rightside();
-        macros.apply(file, "ParserParseTail", new String[] {rightside[0].name});
-      }
+      macros.apply(file, "ParserParseTail", new String[] {firstProductionName});
 
+      //the node creation code. Reduce methods definitions are done here
       for(int i = 0; i < (productions.length - 1); i++)
       {
-        macros.apply(file, "ParserNewHeader",
-                     new String[] {"" + productions[i].index});
+        macros.apply(file, "ParserNewHeader", new String[] {
+                       "" + productions[i].index,
+                       productions[i].name});
 
-        Node node = (Node) alts.get(productions[i].name);
+        final Node node = (Node) alts.get(productions[i].name);
 
-        if(node == null)
-        {
-          Symbol[] rightside = productions[i].rightside();
-          for(int k = rightside.length - 1; k >= 0; k--)
-          {
-            macros.apply(file, "ParserNewBodyDecl",
-                         new String[] {rightside[k].name, "" + (k + 1)});
-          }
+        final BufferedWriter finalFile = file;
+        final LinkedList stack = new LinkedList();
 
-          macros.apply(file, "ParserNewBodyNew",
-                       new String[] {productions[i].name});
+        node.apply(new DepthFirstAdapter()
+                   {
+                     private int current;
 
-          for(int k = 0; k < rightside.length; k++)
-          {
-            macros.apply(file, "ParserNewBodyParams",
-                         new String[] {(k == 0) ? "" : ", ",
-                                       (k + 1) + ""});
-          }
-
-          macros.apply(file, "ParserNewTail", new String[] { "" + productions[i].leftside});
-        }
-        else
-        {
-          count = count(productions[i].name) - 1;
-          elem = 0;
-          final BufferedWriter finalFile = file;
-          final LinkedList stack = new LinkedList();
-
-          node.apply(new DepthFirstAdapter()
+                     public void caseAElem(AElem elem)
                      {
-                       private int current;
+                       current++;
 
-                       public void caseAElem(AElem node)
-                       {
-                         int op = ((Integer) GenParser.this.getOut(node)).intValue();
-                         String name = (String) ids.elemTypes.get(node);
-                         current++;
-
-                         switch(op)
-                         {
-                         case NONE:
-                           {
-                             stack.addFirst(new Element("ParserNewBodyDecl",
-                                                        new String[] {name, "" + current}));
-                           }
-                           break;
-                         case STAR:
-                           {
-                             if((count & (1 << elem)) != 0)
-                             {
-                               stack.addFirst(new Element("ParserNewBodyDecl",
-                                                          new String[] {"X" + name, "" + current}));
-                             }
-                             else
-                             {
-                               stack.addFirst(new Element("ParserNewBodyDeclNull",
-                                                          new String[] {"X" + name, "" + current}));
-                             }
-
-                             elem++;
-                           }
-                           break;
-                         case QMARK:
-                           {
-                             if((count & (1 << elem)) != 0)
-                             {
-                               stack.addFirst(new Element("ParserNewBodyDecl",
-                                                          new String[] {name, "" + current}));
-                             }
-                             else
-                             {
-                               stack.addFirst(new Element("ParserNewBodyDeclNull",
-                                                          new String[] {name, "" + current}));
-                             }
-
-                             elem++;
-                           }
-                           break;
-                         case PLUS:
-                           {
-                             stack.addFirst(new Element("ParserNewBodyDecl",
-                                                        new String[] {"X" + name, "" + current}));
-                           }
-                           break;
-                         }
-                       }
+                       stack.addFirst(new Element("ParserNewBodyDecl",
+                                                  new String[] {"" + current}));
                      }
-                    );
+                   }
+                  );
 
-          try
+        try
+        {
+          for(Iterator it = stack.iterator(); it.hasNext();)
           {
-            for(Iterator it = stack.iterator(); it.hasNext();)
-            {
-              Element e = (Element) it.next();
-
-              macros.apply(file, e.macro, e.arguments);
-            }
+            Element e = (Element) it.next();
+            macros.apply(file, e.macro, e.arguments);
           }
-          catch(IOException e)
-          {
-            throw new RuntimeException("An error occured while writing to " +
-                                       new File(pkgDir, "Parser.java").getAbsolutePath());
-          }
-
-          macros.apply(file, "ParserNewBodyNew",
-                       new String[] {
-                         (node == null) ? productions[i].name : name(productions[i].name)});
-
-          node.apply(new DepthFirstAdapter()
-                     {
-                       private int current;
-
-                       public void caseAElem(AElem node)
-                       {
-                         try
-                         {
-                           String name = (String) ids.elemTypes.get(node);
-                           current++;
-
-                           macros.apply(finalFile, "ParserNewBodyParams",
-                                        new String[] {(current == 1) ? "" : ", ",
-                                                      current + ""});
-                         }
-                         catch(IOException e)
-                         {
-                           throw new RuntimeException("An error occured while writing to " +
-                                                      new File(pkgDir, "Parser.java").getAbsolutePath());
-                         }
-                       }
-                     }
-                    );
-
-          macros.apply(file, "ParserNewTail", new String[] { "" + productions[i].leftside});
         }
+        catch(IOException e)
+        {
+          throw new RuntimeException("An error occured while writing to " +
+                                     new File(pkgDir, "Parser.java").getAbsolutePath());
+        }
+
+        String nodeName = (String)ids.names.get(node);
+        String realnodeName = (String)ids.names.get(node);
+        aParsedAltAdapter =
+          new GenerateAlternativeCodeForParser(pkgDir, nodeName, realnodeName,
+                                               file, transformIds, CG, CTP,
+                                               simpleTermTransform, macros,
+                                               listSimpleTermTransform,
+                                               transformIds.simpleTermOrsimpleListTermTypes);
+        node.apply(aParsedAltAdapter);
       }
 
       macros.apply(file, "ParserActionHeader");
@@ -474,7 +473,7 @@ public class GenParser extends DepthFirstAdapter
                                    new File(pkgDir, "parser.dat"))));
 
       Vector outerArray = new Vector();
-
+      //Generating of paring tables
       for(int i = 0; i < Grammar.action_.length; i++)
       {
         Vector innerArray = new Vector();
@@ -890,4 +889,3 @@ public class GenParser extends DepthFirstAdapter
     }
   }
 }
-
