@@ -70,11 +70,11 @@ public class GenTokens extends DepthFirstAdapter
 
     ARegExp regExp = (ARegExp) node.getRegExp();
 
-    AConcat concat = (AConcat) regExp.getConcat();
-    LinkedList regExpTails = regExp.getConcats();
+    LinkedList concats = regExp.getConcats();
 
-    if(regExpTails.size() == 0)
+    if(concats.size() == 1)
     {
+      AConcat concat = (AConcat)concats.getFirst();
       LinkedList unExps = concat.getUnExps();
 
       if(unExps.size() == 1)
