@@ -15,14 +15,14 @@ import java.io.*;
 public class GenUtils extends DepthFirstAdapter
 {
   private MacroExpander macros;
-  private ResolveIds ids;
+  private ResolveAstIds ast_ids;
   private File pkgDir;
   private String pkgName;
   private String mainProduction;
 
-  public GenUtils(ResolveIds ids)
+  public GenUtils(ResolveAstIds ast_ids)
   {
-    this.ids = ids;
+    this.ast_ids = ast_ids;
 
     try
     {
@@ -35,8 +35,8 @@ public class GenUtils extends DepthFirstAdapter
       throw new RuntimeException("unable to open utils.txt.");
     }
 
-    pkgDir = new File(ids.pkgDir, "node");
-    pkgName = ids.pkgName.equals("") ? "node" : ids.pkgName + ".node";
+    pkgDir = new File(ast_ids.astIds.pkgDir, "node");
+    pkgName = ast_ids.astIds.pkgName.equals("") ? "node" : ast_ids.astIds.pkgName + ".node";
 
     if(!pkgDir.exists())
     {
@@ -47,17 +47,23 @@ public class GenUtils extends DepthFirstAdapter
     }
   }
 
+  /*
   public void caseAGrammar(AGrammar node)
   {
-    if(node.getProductions() != null)
-      node.getProductions().apply(this);
-  }
+  AAstProd []temp = (AAstProd[])node.getAst().toArray(new AAstProd[0]);
 
-  public void caseAProd(AProd node)
+  for(int i=0; i<temp.length; i++)
+  {
+    temp[i].apply(this);
+  }
+  }
+  */
+
+  public void caseAAstProd(AAstProd node)
   {
     if(mainProduction == null)
     {
-      mainProduction = (String) ids.names.get(node);
+      mainProduction = (String) ast_ids.ast_names.get(node);
     }
   }
 
@@ -70,13 +76,9 @@ public class GenUtils extends DepthFirstAdapter
 
     createEOF();
     createNode();
-    createNodeCast();
     createToken();
     create("Switch");
     create("Switchable");
-    create("TypedLinkedList");
-    create("Cast");
-    create("NoCast");
   }
 
   public void createStart()
@@ -97,7 +99,7 @@ public class GenUtils extends DepthFirstAdapter
     try
     {
       macros.apply(file, "Start", new String[] {pkgName,
-                   ids.pkgName.equals("") ? "analysis" : ids.pkgName + ".analysis",
+                   ast_ids.astIds.pkgName.equals("") ? "analysis" : ast_ids.astIds.pkgName + ".analysis",
                    mainProduction, GenAlts.nodeName(mainProduction)});
     }
     catch(IOException e)
@@ -132,7 +134,7 @@ public class GenUtils extends DepthFirstAdapter
     try
     {
       macros.apply(file, "EOF", new String[] {pkgName,
-                                              ids.pkgName.equals("") ? "analysis" : ids.pkgName + ".analysis"});
+                                              ast_ids.astIds.pkgName.equals("") ? "analysis" : ast_ids.astIds.pkgName + ".analysis"});
     }
     catch(IOException e)
     {
@@ -166,7 +168,7 @@ public class GenUtils extends DepthFirstAdapter
     try
     {
       macros.apply(file, "Node", new String[] {pkgName,
-                   ids.pkgName.equals("") ? "analysis" : ids.pkgName + ".analysis"});
+                   ast_ids.astIds.pkgName.equals("") ? "analysis" : ast_ids.astIds.pkgName + ".analysis"});
     }
     catch(IOException e)
     {
@@ -182,39 +184,6 @@ public class GenUtils extends DepthFirstAdapter
     {}
   }
 
-  public void createNodeCast()
-  {
-    BufferedWriter file;
-
-    try
-    {
-      file = new BufferedWriter(
-               new FileWriter(
-                 new File(pkgDir, "NodeCast.java")));
-    }
-    catch(IOException e)
-    {
-      throw new RuntimeException("Unable to create " + new File(pkgDir, "NodeCast.java").getAbsolutePath());
-    }
-
-    try
-    {
-      macros.apply(file, "NodeCast", new String[] {pkgName});
-    }
-    catch(IOException e)
-    {
-      throw new RuntimeException("An error occured while writing to " +
-                                 new File(pkgDir, "NodeCast.java").getAbsolutePath());
-    }
-
-    try
-    {
-      file.close();
-    }
-    catch(IOException e)
-    {}
-  }
-
   public void createToken()
   {
     BufferedWriter file;
@@ -281,4 +250,3 @@ public class GenUtils extends DepthFirstAdapter
     {}
   }
 }
-
