@@ -34,26 +34,40 @@ public class ResolveIds extends DepthFirstAdapter
                              StringComparator.instance,
                              StringCast.instance,
                              NodeCast.instance);
-  public final Map alts = new TypedTreeMap(
-                            StringComparator.instance,
+
+  public final Map alts = new TypedHashMap(
                             StringCast.instance,
                             NodeCast.instance);
-  public final Map elems = new TypedTreeMap(
-                             StringComparator.instance,
+
+  public final Map elems = new TypedHashMap(
                              StringCast.instance,
                              NodeCast.instance);
+
   public final Map names = new TypedHashMap(
                              NodeCast.instance,
                              StringCast.instance);
+
   public final Map errorNames = new TypedHashMap(
                                   NodeCast.instance,
                                   StringCast.instance);
   public final Map elemTypes = new TypedHashMap(
                                  NodeCast.instance,
                                  StringCast.instance);
+
+  public final Map altsElemNameTypes = new TypedHashMap(
+                                         StringCast.instance,
+                                         StringCast.instance);
+
+  // This map will serve for simpleTerm and simplelistTerm type within an altTransform
+  // Inside an altTransform, one would look at this map to know its type. (P... or T...)
+  public final Map altsElemTypes = new TypedHashMap(
+                                     StringCast.instance,
+                                     StringCast.instance);
+
   public final Map fixedTokens = new TypedHashMap(
                                    NodeCast.instance,
                                    BooleanCast.instance);
+
   public final List tokenList = new TypedLinkedList(StringCast.instance);
   public final LinkedList stateList = new TypedLinkedList(StringCast.instance);
   public File pkgDir;
@@ -72,31 +86,48 @@ public class ResolveIds extends DepthFirstAdapter
     pkgDir = currentDir;
   }
 
-  public void inAPkgName(APkgName node)
+  public void inAGrammar(AGrammar node)
   {
-    pkgName = node.getPkgId().getText();
-    pkgDir = new File(pkgDir, node.getPkgId().getText());
-
-    if(!pkgDir.exists())
+    TPkgId[] temp = (TPkgId []) node.getPackage().toArray(new TPkgId[0]);
+    if(temp.length > 0)
     {
-      if(!pkgDir.mkdir())
+      pkgName = temp[0].getText();
+      pkgDir = new File(pkgDir, temp[0].getText());
+
+      for(int i=1; i<temp.length; i++)
       {
-        throw new RuntimeException("Unable to create " + pkgDir.getAbsolutePath());
+        pkgName += "." + temp[i].getText();
+        pkgDir = new File(pkgDir, temp[i].getText());
+      }
+
+      if(!pkgDir.exists())
+      {
+        if(!pkgDir.mkdirs())
+        {
+          throw new RuntimeException("Unable to create " + pkgDir.getAbsolutePath());
+        }
       }
     }
   }
 
-  public void inAPkgNameTail(APkgNameTail node)
+  public void caseAProd(AProd node)
   {
-    pkgName += "." + node.getPkgId().getText();
-    pkgDir = new File(pkgDir, node.getPkgId().getText());
+    //inAProd code.
+    currentProd = name(node.getId().getText());
+    String name = "P" + currentProd;
 
-    if(!pkgDir.exists())
+    if(prods.put(name, node) != null)
     {
-      if(!pkgDir.mkdir())
-      {
-        throw new RuntimeException("Unable to create " + pkgDir.getAbsolutePath());
-      }
+      error(node.getId(), name);
+    }
+
+    names.put(node, name);
+
+    //list of inAAlt code.
+    Object []list_alt = (Object [])node.getAlts().toArray();
+    for(int i = 0; i< list_alt.length; i++)
+    {
+      ((PAlt)list_alt[i]).apply(this);
     }
   }
 
@@ -104,6 +135,7 @@ public class ResolveIds extends DepthFirstAdapter
   {
     String name = node.getId().getText();
 
+    // Only helpers can be used inside tokens definition
     if(helpers.get(name) == null)
     {
       error2(node.getId(), name);
@@ -114,6 +146,8 @@ public class ResolveIds extends DepthFirstAdapter
   {
     String name = node.getId().getText();
 
+    // If another helper is used within the current helper,
+    // it should have been defined before the current one
     if(helpers.put(name, node) != null)
     {
       error(node.getId(), name);
@@ -127,6 +161,7 @@ public class ResolveIds extends DepthFirstAdapter
     String name = "T" + name(node.getId().getText());
     String errorName = errorName(node.getId().getText());
 
+    //We are making sure that this token is not yet defined.
     if(tokens.put(name, node) != null)
     {
       error(node.getId(), name);
@@ -135,91 +170,54 @@ public class ResolveIds extends DepthFirstAdapter
     names.put(node, name);
     errorNames.put(node, errorName);
     tokenList.add(name);
+
+    if(node.getLookAhead() != null)
+    {
+      Token token = (Token) node.getSlash();
+      throw new RuntimeException(
+        "[" + token.getLine() + "," + token.getPos() + "] " +
+        "Look ahead not yet supported.");
+    }
   }
 
   public void inAStates(AStates node)
   {
-    processingStates = true;
-  }
+    Object [] list_id = (Object[]) node.getListId().toArray();
+    String name;
 
-  public void outAStates(AStates node)
-  {
-    processingStates = false;
+    for(int i=0; i<list_id.length; i++)
+    {
+      name = ((TId)list_id[i]).getText().toUpperCase();
+
+      if(states.put(name, list_id[i]) != null)
+      {
+        error((TId)list_id[i], name);
+      }
+
+      names.put(list_id[i], name);
+      stateList.add(name);
+    }
   }
 
   public void inAIgnTokens(AIgnTokens node)
   {
-    processingIgnTokens = true;
-  }
+    Object [] list_id = (Object[]) node.getListId().toArray();
+    String name;
 
-  public void outAIgnTokens(AIgnTokens node)
-  {
-    processingIgnTokens = false;
-  }
-
-  public void inAIdList(AIdList node)
-  {
-    if(processingStates)
+    for(int i=0; i<list_id.length; i++)
     {
-      String name = node.getId().getText().toUpperCase();
-
-      if(states.put(name, node.getId()) != null)
-      {
-        error(node.getId(), name);
-      }
-
-      names.put(node.getId(), name);
-      stateList.add(name);
-    }
-
-    if(processingIgnTokens)
-    {
-      String name = "T" + name(node.getId().getText());
+      name = "T" + name(((TId)list_id[i]).getText());
 
       if(tokens.get(name) == null)
       {
-        error2(node.getId(), name);
+        error2((TId)list_id[i], name);
       }
 
-      if(ignTokens.put(name, node.getId()) != null)
+      if(ignTokens.put(name, list_id[i]) != null)
       {
-        error(node.getId(), name);
+        error((TId)list_id[i], name);
       }
-
-      names.put(node.getId(), name);
-    }
-  }
-
-  public void inAIdListTail(AIdListTail node)
-  {
-    if(processingStates)
-    {
-      String name = node.getId().getText().toUpperCase();
-
-      if(states.put(name, node.getId()) != null)
-      {
-        error(node.getId(), name);
-      }
-
-      names.put(node.getId(), name);
-      stateList.add(name);
-    }
-
-    if(processingIgnTokens)
-    {
-      String name = "T" + name(node.getId().getText());
-
-      if(tokens.get(name) == null)
-      {
-        error2(node.getId(), name);
-      }
-
-      if(ignTokens.put(name, node.getId()) != null)
-      {
-        error(node.getId(), name);
-      }
-
-      names.put(node.getId(), name);
+      names.put(list_id[i], name);
     }
   }
 
@@ -275,105 +273,73 @@ public class ResolveIds extends DepthFirstAdapter
     }
   }
 
-  public void inAProd(AProd node)
-  {
-    currentProd = name(node.getId().getText());
-
-    String name = "P" + currentProd;
-
-    if(prods.put(name, node) != null)
-    {
-      error(node.getId(), name);
-    }
-
-    names.put(node, name);
-  }
-
-  public void inAParsedAlt(AParsedAlt alt)
+  public void caseAAlt(final AAlt alt)
   {
     if(alt.getAltName() != null)
     {
       currentAlt =
         "A" +
-        name(((AAltName) alt.getAltName()).getId().getText()) +
+        name(alt.getAltName().getText()) +
         currentProd;
 
       if(alts.put(currentAlt, alt) != null)
       {
-        error(((AAltName) alt.getAltName()).getId(), currentAlt);
+        error(alt.getAltName(), currentAlt);
       }
 
       names.put(alt, currentAlt);
     }
     else
     {
+
       currentAlt = "A" + currentProd;
 
       if(alts.put(currentAlt, alt) != null)
       {
         error(currentAlt);
       }
-
       names.put(alt, currentAlt);
     }
-  }
 
-  public void inAIgnoredAlt (AIgnoredAlt  alt)
-  {
-    if(alt.getAltName() != null)
+    AElem list_elem[] = (AElem[]) alt.getElems().toArray(new AElem[0]);
+    for(int i=0; i<list_elem.length;i++)
     {
-      currentAlt =
-        "A" +
-        name(((AAltName) alt.getAltName()).getId().getText()) +
-        currentProd;
-
-      if(alts.put(currentAlt, alt) != null)
-      {
-        error(((AAltName) alt.getAltName()).getId(), currentAlt);
-      }
-
-      names.put(alt, currentAlt);
+      list_elem[i].apply(this);
     }
-    else
-    {
-      currentAlt = "A" + currentProd;
 
-      if(alts.put(currentAlt, alt) != null)
-      {
-        error(currentAlt);
-      }
-
-      names.put(alt, currentAlt);
-    }
   }
 
-  public void inALookAhead(ALookAhead node)
+  public void defaultcase(Node node)
   {
-    Token token = (Token) node.getSlash();
-
-    throw new RuntimeException(
-      "[" + token.getLine() + "," + token.getPos() + "] " +
-      "Look ahead not yet supported.");
+    if(node instanceof Token)
+    {
+      Token t = (Token) node;
+      lastLine = t.getLine();
+      lastPos = t.getPos() + t.getText().length();
+    }
   }
 
-  public void caseAElem(AElem elem)
+  public void caseAAst(AAst node)
+  {}
+
+  public void caseAElem(final AElem elem)
   {
     if(elem.getElemName() != null)
     {
       String name = currentAlt + "." +
-                    name(((AElemName)elem.getElemName()).getId().getText());
+                    name(elem.getElemName().getText());
 
       if(elems.put(name, elem) != null)
       {
-        error(((AElemName)elem.getElemName()).getId(), name);
+        error(elem.getElemName(), name);
       }
 
-      if(((AElemName)elem.getElemName()).getId().getText().equals("class"))
+      if(elem.getElemName().getText().equals("class"))
       {
-        error5(((AElemName)elem.getElemName()).getId());
+        error5(elem.getElemName());
       }
 
-      names.put(elem, name(((AElemName)elem.getElemName()).getId().getText()));
+      names.put(elem, name(elem.getElemName().getText()) );
     }
     else
     {
@@ -398,9 +364,50 @@ public class ResolveIds extends DepthFirstAdapter
   {
     prod.apply(new DepthFirstAdapter()
                {
+                 public void caseAProd(AProd node)
+                 {
+                   //inAProd code.
+                   currentProd = name(node.getId().getText());
+
+                   //list of inAAlt code.
+                   Object []list_alt = (Object [])node.getAlts().toArray();
+                   for(int i = 0; i< list_alt.length; i++)
+                   {
+                     ((PAlt)list_alt[i]).apply(this);
+                   }
+                 }
+
+                 public void caseAAlt(final AAlt alt)
+                 {
+                   if(alt.getAltName() != null)
+                   {
+                     currentAlt = "A" + name(alt.getAltName().getText()) + currentProd;
+                   }
+                   else
+                   {
+                     currentAlt = "A" + currentProd;
+                   }
+
+                   AElem[] list_elem = (AElem[]) alt.getElems().toArray(new AElem[0]);
+                   for(int i=0; i<list_elem.length;i++)
+                   {
+                     list_elem[i].apply(this);
+                   }
+                 }
+
                  public void caseAElem(AElem node)
                  {
                    String name = name(node.getId().getText());
+                   String nameOfElem;
+
+                   if(node.getElemName() != null)
+                   {
+                     nameOfElem = node.getElemName().getText();
+                   }
+                   else
+                   {
+                     nameOfElem = node.getId().getText();
+                   }
 
                    if(node.getSpecifier() != null)
                    {
@@ -417,6 +424,18 @@ public class ResolveIds extends DepthFirstAdapter
                        }
 
                        elemTypes.put(node, "T" + name);
+
+                       if(node.getElemName() != null)
+                       {
+                         altsElemNameTypes.put(currentAlt+"." + node.getElemName().getText(), "T" + name);
+                       }
+
+                       String type_name = name;
+                       if(node.getUnOp() instanceof AStarUnOp || node.getUnOp() instanceof AQMarkUnOp)
+                       {
+                         type_name += "?";
+                       }
+                       altsElemTypes.put(currentAlt+"." + nameOfElem, "T" + type_name);
                      }
                      else
                      {
@@ -426,6 +445,18 @@ public class ResolveIds extends DepthFirstAdapter
                        }
 
                        elemTypes.put(node, "P" + name);
+
+                       if(node.getElemName() != null)
+                       {
+                         altsElemNameTypes.put(currentAlt+"." + node.getElemName().getText(), "P" + name);
+                       }
+                       //altsElemTypes.put(currentAlt+"." + nameOfElem, "P" + name);
+                       String type_name = name;
+                       if(node.getUnOp() instanceof AStarUnOp || node.getUnOp() instanceof AQMarkUnOp)
+                       {
+                         type_name += "?";
+                       }
+                       altsElemTypes.put(currentAlt+"." + nameOfElem, "P" + type_name);
                      }
                    }
                    else
@@ -452,10 +483,32 @@ public class ResolveIds extends DepthFirstAdapter
                        }
 
                        elemTypes.put(node, "T" + name);
+
+                       if(node.getElemName() != null)
+                       {
+                         altsElemNameTypes.put(currentAlt+"." + node.getElemName().getText(), "T" + name);
+                       }
+                       String type_name = name;
+                       if(node.getUnOp() instanceof AStarUnOp || node.getUnOp() instanceof AQMarkUnOp)
+                       {
+                         type_name += "?";
+                       }
+                       altsElemTypes.put(currentAlt+"." + nameOfElem, "T" + type_name);
                      }
                      else
                      {
                        elemTypes.put(node, "P" + name);
+
+                       if(node.getElemName() != null)
+                       {
+                         altsElemNameTypes.put(currentAlt+"." + node.getElemName().getText(), "P" + name);
+                       }
+                       String type_name = name;
+                       if(node.getUnOp() instanceof AStarUnOp || node.getUnOp() instanceof AQMarkUnOp)
+                       {
+                         type_name += "?";
+                       }
+                       altsElemTypes.put(currentAlt+"." + nameOfElem, "P" + type_name);
                      }
                    }
                  }
@@ -463,16 +516,6 @@ public class ResolveIds extends DepthFirstAdapter
               );
   }
 
-  public void defaultcase(Node node)
-  {
-    if(node instanceof Token)
-    {
-      Token t = (Token) node;
-      lastLine = t.getLine();
-      lastPos = t.getPos() + t.getText().length();
-    }
-  }
-
   public static String name(String s)
   {
     StringBuffer result = new StringBuffer();
@@ -488,6 +531,10 @@ public class ResolveIds extends DepthFirstAdapter
       case '_':
         upcase = true;
         break;
+      case '$':
+        result.append(c);
+        upcase = true;
+        break;
       default:
         if(upcase)
         {
@@ -532,6 +579,12 @@ public class ResolveIds extends DepthFirstAdapter
     return result.toString();
   }
 
+  public void reinit()
+  {
+    names.clear();
+    elemTypes.clear();
+  }
+
   private static void error(Token token, String name)
   {
     throw new RuntimeException(
@@ -617,4 +670,3 @@ public class ResolveIds extends DepthFirstAdapter
     return s.toString();
   }
 }
-
