@@ -18,6 +18,16 @@ import java.util.Vector;
 
 public class SableCC
 {
+  private static boolean processInlining = true;
+  static int inliningMaxAlts = 20;
+  private static boolean prettyPrinting = false;
+
+  private static final String OPT_LICENSE = "--license";
+  private static final String OPT_D = "-d";
+  private static final String OPT_NO_INLINE = "--no-inline";
+  private static final String OPT_INLINE_MAX_ALTS = "--inline-max-alts";
+  private static final String OPT_PRETTY_PRINT = "--pretty-print";
+
   private static void displayCopyright()
   {
     System.out.println();
@@ -36,8 +46,12 @@ public class SableCC
   private static void displayUsage()
   {
     System.out.println("Usage:");
-    System.out.println("  sablecc [-d destination] filename [filename]...");
-    System.out.println("  sablecc -license");
+    System.out.println("  sablecc [" +
+                       OPT_D + " destination] [" +
+                       OPT_NO_INLINE + "] [" +
+                       OPT_INLINE_MAX_ALTS + " number] [" +
+                       OPT_PRETTY_PRINT + "] filename [filename]...");
+    System.out.println("  sablecc " + OPT_LICENSE);
   }
 
   public static void main(String[] arguments)
@@ -52,7 +66,7 @@ public class SableCC
       System.exit(1);
     }
 
-    if((arguments.length == 1) && (arguments[0].equals("-license")))
+    if((arguments.length == 1) && (arguments[0].equals(OPT_LICENSE)))
     {
       new DisplayLicense();
       System.exit(0);
@@ -64,7 +78,7 @@ public class SableCC
       int arg = 0;
       while(arg < arguments.length)
       {
-        if(arguments[arg].equals("-d"))
+        if(arguments[arg].equals(OPT_D))
         {
           if((d_option == null) && (++arg < arguments.length))
           {
@@ -76,6 +90,32 @@ public class SableCC
             System.exit(1);
           }
         }
+        else if(arguments[arg].equals(OPT_NO_INLINE))
+        {
+          processInlining = false;
+        }
+        /* A production is not inlined if it has more than
+         * inliningMaxAlts alternatives.  The default value is 20. */
+        else if(arguments[arg].equals(OPT_INLINE_MAX_ALTS))
+        {
+          try
+          {
+            inliningMaxAlts = Integer.parseInt(arguments[++arg]);
+          }
+          catch(Exception e)
+          {
+            displayUsage();
+            System.exit(1);
+          }
+        }
+        /*
+          if prettyprint flag is set to true, only the transformed
+          grammar is printed on standard output
+        */
+        else if(arguments[arg].equals(OPT_PRETTY_PRINT))
+        {
+          prettyPrinting = true;
+        }
         else
         {
           filename.addElement(arguments[arg]);
@@ -99,26 +139,18 @@ public class SableCC
     }
     catch(Exception e)
     {
-      System.out.println(e);
+      e.printStackTrace();
       System.exit(1);
     }
-    catch(Throwable e)
-    {
-      System.out.println(e);
-      System.exit(1);
-    }
-    finally
-    {
-      System.exit(0);
-    }
+    System.exit(0);
   }
 
   /**
    * The main method for processing grammar file and generating the parser/lexer.
-   * @param in input grammar file
-   * @param dir output directory
+   * @param grammar input grammar file name
+   * @param destDir output directory name
    */
-  public static void processGrammar(String grammar, String destDir) throws Exception, Throwable
+  public static void processGrammar(String grammar, String destDir) throws Exception
   {
     File in;
     File dir;
@@ -144,7 +176,7 @@ public class SableCC
    * @param in input grammar file
    * @param dir output directory
    */
-  public static void processGrammar(File in,  File dir) throws Exception, Throwable
+  public static void processGrammar(File in,  File dir) throws Exception
   {
     if(!in.exists())
     {
@@ -165,32 +197,71 @@ public class SableCC
 
     System.out.println("\n -- Generating parser for "+in.getName()+" in "+dir.getPath());
 
-    FileReader temp;
+    FileReader temp = new FileReader(in);
 
     // Build the AST
     Start tree = new Parser(new Lexer(new PushbackReader(
-                                        new BufferedReader(temp = new FileReader(in)), 1000))).parse();
+                                        temp = new FileReader(in), 1000))).parse();
+
     temp.close();
 
+    boolean hasTransformations = false;
+
+    if( ((AGrammar)tree.getPGrammar()).getAst() == null )
+    {
+      System.out.println("Adding productions and alternative of section AST.");
+      //AddAstProductions astProductions = new AddAstProductions();
+      tree.apply(new AddAstProductions());
+    }
+    else
+    {
+      hasTransformations = true;
+    }
+
     System.out.println("Verifying identifiers.");
     ResolveIds ids = new ResolveIds(dir);
     tree.apply(ids);
 
-    // Create the node.* and analysis.* files
+    System.out.println("Verifying ast identifiers.");
+    ResolveAstIds ast_ids = new ResolveAstIds(ids);
+    tree.apply(ast_ids);
+
+    System.out.println("Adding empty productions and empty alternative transformation if necessary.");
+    tree.apply( new AddEventualEmptyTransformationToProductions(ids, ast_ids) );
+
+    System.out.println("Adding productions and alternative transformation if necessary.");
+    AddProdTransformAndAltTransform adds = new AddProdTransformAndAltTransform();
+    tree.apply(adds);
+    /*
+    System.out.println("Replacing AST + operator by * and removing ? operator if necessary");
+    tree.apply( new AstTransformations() );
+    */
+    System.out.println("computing alternative symbol table identifiers.");
+    ResolveAltIds alt_ids = new ResolveAltIds(ids);
+    tree.apply(alt_ids);
+
+    System.out.println("Verifying production transform identifiers.");
+    ResolveProdTransformIds ptransform_ids = new ResolveProdTransformIds(ast_ids);
+    tree.apply(ptransform_ids);
+
+    System.out.println("Verifying ast alternatives transform identifiers.");
+    ResolveTransformIds transform_ids = new ResolveTransformIds(ast_ids, alt_ids, ptransform_ids);
+    tree.apply(transform_ids);
+
     System.out.println("Generating token classes.");
     tree.apply(new GenTokens(ids));
 
     System.out.println("Generating production classes.");
-    tree.apply(new GenProds(ids));
+    tree.apply(new GenProds(ast_ids));
 
     System.out.println("Generating alternative classes.");
-    tree.apply(new GenAlts(ids));
+    tree.apply(new GenAlts(ast_ids));
 
     System.out.println("Generating analysis classes.");
-    tree.apply(new GenAnalyses(ids));
+    tree.apply(new GenAnalyses(ast_ids));
 
     System.out.println("Generating utility classes.");
-    tree.apply(new GenUtils(ids));
+    tree.apply(new GenUtils(ast_ids));
 
     try
     {
@@ -206,7 +277,7 @@ public class SableCC
     try
     {
       System.out.println("Generating the parser.");
-      tree.apply(new GenParser(ids));
+      tree.apply(new GenParser(ids, alt_ids, transform_ids, ast_ids.getFirstAstProduction(), processInlining, prettyPrinting, hasTransformations) );
     }
     catch(Exception e)
     {
@@ -214,6 +285,4 @@ public class SableCC
       throw e;
     }
   }
-
 }
-
