@@ -24,10 +24,6 @@ public class Transitions extends DepthFirstAdapter
   public void caseAStateList(AStateList node)
   {
     inAStateList(node);
-    if(node.getLBrace() != null)
-    {
-      node.getLBrace().apply(this);
-    }
     if(node.getId() != null)
     {
       node.getId().apply(this);
@@ -46,10 +42,6 @@ public class Transitions extends DepthFirstAdapter
         ((PStateListTail) temp[i]).apply(this);
       }
     }
-    if(node.getRBrace() != null)
-    {
-      node.getRBrace().apply(this);
-    }
   }
 
   public void inATokenDef(ATokenDef node)
