@@ -31,6 +31,21 @@ public class TypedTreeMap extends TreeMap
     valueCast = NoCast.instance;
   }
 
+  public TypedTreeMap(Map map)
+  {
+    super();
+
+    keyCast = NoCast.instance;
+    valueCast = NoCast.instance;
+
+    Map.Entry[] entries = (Map.Entry[])map.entrySet().toArray(new Map.Entry[0]);
+    for(int i=0; i < entries.length; i++)
+    {
+      this.put(entries[i].getKey(),entries[i].getValue());
+    }
+  }
+
+  /*
   public TypedTreeMap(Map map)
   {
     super(map);
@@ -38,7 +53,23 @@ public class TypedTreeMap extends TreeMap
     keyCast = NoCast.instance;
     valueCast = NoCast.instance;
   }
+  */
 
+  public TypedTreeMap(SortedMap smap)
+  {
+    super(smap.comparator());
+
+    keyCast = NoCast.instance;
+    valueCast = NoCast.instance;
+
+    Map.Entry[] entries = (Map.Entry[])smap.entrySet().toArray(new Map.Entry[0]);
+    for(int i=0; i < entries.length; i++)
+    {
+      this.put(entries[i].getKey(),entries[i].getValue());
+    }
+
+  }
+  /*
   public TypedTreeMap(SortedMap smap)
   {
     super(smap);
@@ -46,6 +77,7 @@ public class TypedTreeMap extends TreeMap
     keyCast = NoCast.instance;
     valueCast = NoCast.instance;
   }
+  */
 
   public TypedTreeMap(Cast keyCast, Cast valueCast)
   {
@@ -75,15 +107,24 @@ public class TypedTreeMap extends TreeMap
     this.keyCast = keyCast;
     this.valueCast = valueCast;
 
-    Iterator entryIter = map.entrySet().iterator();
-    while (entryIter.hasNext())
+    Map.Entry[] entries = (Map.Entry[])map.entrySet().toArray(new Map.Entry[0]);
+    for(int i=0; i < entries.length; i++)
     {
-      Map.Entry entry = (Map.Entry) entryIter.next();
-      this.put(entry.getKey(),entry.getValue());
+      this.put(entries[i].getKey(),entries[i].getValue());
     }
 
   }
 
+  /*
+    public TypedTreeMap(Map map, Cast keyCast, Cast valueCast)
+    {
+      super(map);
+
+      this.keyCast = keyCast;
+      this.valueCast = valueCast;
+    }
+  */
+
   public TypedTreeMap(SortedMap smap, Cast keyCast, Cast valueCast)
   {
     super(smap.comparator());
@@ -91,15 +132,24 @@ public class TypedTreeMap extends TreeMap
     this.keyCast = keyCast;
     this.valueCast = valueCast;
 
-    Iterator entryIter = smap.entrySet().iterator();
-    while (entryIter.hasNext())
+    Map.Entry[] entries = (Map.Entry[])smap.entrySet().toArray(new Map.Entry[0]);
+    for(int i=0; i < entries.length; i++)
     {
-      Map.Entry entry = (Map.Entry) entryIter.next();
-      this.put(entry.getKey(),entry.getValue());
+      this.put(entries[i].getKey(),entries[i].getValue());
     }
 
   }
 
+  /*
+  public TypedTreeMap(SortedMap smap, Cast keyCast, Cast valueCast)
+  {
+    super(smap);
+
+    this.keyCast = keyCast;
+    this.valueCast = valueCast;
+  }
+  */
+
   public Cast getKeyCast()
   {
     return keyCast;
